==> _private_chat-0.0.5.tar.gz: lib/chat_detail/widgets/gif/widget_gif.dart <==
import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:get/get.dart';
import 'package:lottie/lottie.dart';
import 'package:_private_core/_private_core.dart';

import '../widget_message_content.dart';
import 'client/client.dart';
import 'client/models/collection.dart';
import 'client/models/gif.dart';

Color get _backgroundColor => hexColor('#151617');
Color get _element => hexColor('#151617');
Color get _element2 => hexColor('#18191C');

class WidgetGifsResult {
  bool isGif;
  String value;
  WidgetGifsResult(this.isGif, this.value);
}

class WidgetGifs extends StatefulWidget {
  const WidgetGifs({Key? key}) : super(key: key);

  @override
  State<WidgetGifs> createState() => _WidgetGifsState();
}

class _WidgetGifsState extends State<WidgetGifs> {
  bool isGif = true;

  @override
  Widget build(BuildContext context) {
    return Container(
      height: Get.height * .85,
      decoration: BoxDecoration(
        color: _element2,
        borderRadius: const BorderRadius.vertical(top: Radius.circular(16)),
      ),
      padding: const EdgeInsets.fromLTRB(0, 12, 0, 12),
      child: Column(
        children: [
          Container(
            width: 50,
            height: 8,
            decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(4),
                color: _backgroundColor),
          ),
          const SizedBox(
            height: 12,
          ),
          Padding(
            padding: const EdgeInsets.symmetric(horizontal: 24),
            child: Row(
              children: [
                Expanded(
                  child: InkWell(
                    onTap: () {
                      arckipelHaptic();
                      setState(() {
                        isGif = true;
                      });
                    },
                    child: Center(
                      child: Container(
                        padding: const EdgeInsets.symmetric(
                            horizontal: 4, vertical: 2),
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(4),
                            border: Border.all(
                                color: isGif
                                    ? hexColor('#2F3240')
                                    : Colors.transparent)),
                        child: Icon(
                          Icons.gif,
                          size: 32,
                          color: hexColor('#2F3240'),
                        ),
                      ),
                    ),
                  ),
                ),
                Expanded(
                  child: InkWell(
                    onTap: () {
                      arckipelHaptic();
                      setState(() {
                        isGif = false;
                      });
                    },
                    child: Center(
                      child: Container(
                        padding: const EdgeInsets.symmetric(
                            horizontal: 8, vertical: 6),
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(4),
                            border: Border.all(
                                color: isGif
                                    ? Colors.transparent
                                    : hexColor('#2F3240'))),
                        child: Icon(
                          Icons.emoji_emotions,
                          size: 22,
                          color: hexColor('#2F3240'),
                        ),
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ),
          Divider(
            height: 30,
            thickness: 2.5,
            color: _backgroundColor,
          ),
          Expanded(
            child: AnimatedSwitcher(
              duration: const Duration(milliseconds: 200),
              child: isGif ? const _WidgetGifs() : const _WidgetEmojis(),
            ),
          )
        ],
      ),
    );
  }
}

class _WidgetGifs extends StatefulWidget {
  const _WidgetGifs({Key? key}) : super(key: key);

  @override
  __WidgetGifsState createState() => __WidgetGifsState();
}

class __WidgetGifsState extends State<_WidgetGifs> {
  late final ScrollController _scrollController = ScrollController();

  final GiphyClient client = GiphyClient(apiKey: GIPHY_API_KEY, randomId: '');
  final List<dynamic> results = [];

  final Axis _scrollDirection = Axis.vertical;
  late final int _crossAxisCount =
      (MediaQuery.of(context).size.width / _gifWidth).round();
  final double _spacing = 8.0;
  late final double _gifWidth = 200;
  late final int _limit = _crossAxisCount *
          ((MediaQuery.of(context).size.height - 30) / _gifWidth).round() +
      10;
  bool _isFetching = false;
  GiphyCollection? _collectionGiphy;

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      _scrollController.addListener(() {
        if ((_scrollController.position.extentAfter < 200) && !_isFetching) {
          _fetch();
        }
      });
      _fetch();
    });
  }

  _fetch([String? keyword]) async {
    if (_isFetching) return;
    _isFetching = true;
    if (keyword != null && keyword.trim().isNotEmpty) {
      results.clear();
      _collectionGiphy = await client.search(keyword, offset: 0, limit: _limit);
    } else {
      _collectionGiphy =
          await client.trending(offset: results.length, limit: _limit);
    }
    if (mounted) {
      setState(() {
        _isFetching = false;
        List data = [];
        data.addAll(_collectionGiphy?.data ?? []);
        data.shuffle();
        results.addAll(data);
      });
    }
    // results.forEach((element) {
    //   log('Tener result: ${element.toJson()}');
    // });
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.fromLTRB(16, 12, 16, 12),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Container(
            height: 38,
            decoration: BoxDecoration(
              color: _element,
              borderRadius: BorderRadius.circular(99),
            ),
            child: Center(
              child: Row(
                children: [
                  const SizedBox(
                    width: 16,
                  ),
                  Icon(
                    Icons.search,
                    color: hexColor('#2F3240'),
                  ),
                  Expanded(
                    child: TextFormField(
                        maxLines: 1,
                        onChanged: _fetch,
                        style: robotoRegular.copyWith(fontSize: 12),
                        decoration: InputDecoration(
                          isCollapsed: true,
                          hintText: 'searchgif'.tr,
                          contentPadding:
                              const EdgeInsets.symmetric(horizontal: 12),
                          hintStyle: robotoRegular.copyWith(
                              color: hexColor('#2F3240'),
                              fontSize: 12,
                              fontStyle: FontStyle.italic),
                          border: InputBorder.none,
                        )),
                  ),
                ],
              ),
            ),
          ),
          const SizedBox(
            height: 24,
          ),
          Expanded(
            child: _buildGifs(),
          ),
        ],
      ),
    );
  }

  Widget _buildGifs() {
    return results.isEmpty
        ? Center(child: Lottie.asset('assets/lotties/request.json', width: 120))
        : MasonryGridView.count(
            scrollDirection: _scrollDirection,
            controller: _scrollController,
            itemCount: results.length,
            crossAxisCount: _crossAxisCount,
            mainAxisSpacing: _spacing,
            crossAxisSpacing: _spacing,
            itemBuilder: (ctx, index) {
              return _item(index);
            },
            // staggeredTileBuilder: (idx) => StaggeredTile.fit(1),
          );
  }

  Widget _item(int index) {
    GiphyGif gif = results[index];
    double height = double.parse(gif.images!.fixedWidth.height) *
        (_gifWidth / double.parse(gif.images!.fixedWidth.width));
    return ClipRRect(
      borderRadius: BorderRadius.circular(10.0),
      child: InkWell(
        onTap: () {
          arckipelHaptic();
          Navigator.pop(
              context, WidgetGifsResult(true, gif.images!.fixedWidth.webp!));
        },
        child: ExtendedImage.network(gif.images!.fixedWidth.webp!,
            cache: true,
            gaplessPlayback: true,
            fit: BoxFit.fill,
            headers: const {'accept': 'image/*'}, loadStateChanged: (state) {
          return AnimatedSwitcher(
            duration: const Duration(milliseconds: 350),
            child: case2(
                state.extendedImageLoadState,
                {
                  LoadState.loading: Container(
                    color: _backgroundColor,
                    width: _gifWidth,
                    height: height,
                    child: Center(
                      child: Lottie.asset('assets/lotties/gifLoader.json',
                          package: '_private_chat', width: _gifWidth / 2),
                    ),
                  ),
                  LoadState.completed: ExtendedRawImage(
                    image: state.extendedImageInfo?.image,
                    width: _gifWidth,
                    fit: BoxFit.fill,
                  ),
                  LoadState.failed: Container(
                    color: _backgroundColor,
                    width: _gifWidth,
                    height: height,
                    child: const Center(
                      child: Icon(Icons.info),
                    ),
                  ),
                },
                Container(
                  color: _backgroundColor,
                  width: _gifWidth,
                  height: height,
                  child: Center(
                    child: Lottie.asset('assets/lotties/gifLoader.json',
                        package: '_private_chat', width: _gifWidth / 2),
                  ),
                )),
          );
        }),
      ),
    );
  }

  TValue? case2<TOptionType, TValue>(
    TOptionType selectedOption,
    Map<TOptionType, TValue> branches, [
    TValue? defaultValue,
  ]) {
    if (!branches.containsKey(selectedOption)) {
      return defaultValue;
    }

    return branches[selectedOption];
  }
}

class _WidgetEmojis extends StatefulWidget {
  const _WidgetEmojis({Key? key}) : super(key: key);

  @override
  __WidgetEmojisState createState() => __WidgetEmojisState();
}

class __WidgetEmojisState extends State<_WidgetEmojis> {
  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.fromLTRB(16, 12, 16, 12),
      child: GridView.count(
        crossAxisCount: 5,
        mainAxisSpacing: 20,
        crossAxisSpacing: 20,
        children: chatEmojis2
            .map((e) => GestureDetector(
                onTap: () {
                  Navigator.pop(context, WidgetGifsResult(false, e));
                },
                child: Lottie.asset(assetlottie('emoji2/$e'),
                    package: '_private_chat')))
            .toList(),
      ),
    );
  }
}

==> _private_chat-0.0.20.tar.gz: lib/chat_detail/widgets/gif/widget_gif.dart <==
import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:get/get.dart';
import 'package:lottie/lottie.dart';
import 'package:_private_core/_private_core.dart';

import '../widget_message_content.dart';
import 'client/client.dart';
import 'client/models/collection.dart';
import 'client/models/gif.dart';

Color get _backgroundColor => hexColor('#151617');
Color get _element => hexColor('#151617');
Color get _element2 => hexColor('#18191C');

class WidgetGifsResult {
  bool isGif;
  String value;
  WidgetGifsResult(this.isGif, this.value);
}

class WidgetGifs extends StatefulWidget {
  const WidgetGifs({Key? key}) : super(key: key);

  @override
  State<WidgetGifs> createState() => _WidgetGifsState();
}

class _WidgetGifsState extends State<WidgetGifs> {
  bool isGif = true;

  @override
  Widget build(BuildContext context) {
    return Container(
      height: Get.height * .85,
      decoration: BoxDecoration(
        color: _element2,
        borderRadius: const BorderRadius.vertical(top: Radius.circular(16)),
      ),
      padding: const EdgeInsets.fromLTRB(0, 12, 0, 12),
      child: Column(
        children: [
          Container(
            width: 50,
            height: 8,
            decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(4),
                color: _backgroundColor),
          ),
          const SizedBox(
            height: 12,
          ),
          Padding(
            padding: const EdgeInsets.symmetric(horizontal: 24),
            child: Row(
              children: [
                Expanded(
                  child: InkWell(
                    onTap: () {
                      arckipelHaptic();
                      setState(() {
                        isGif = true;
                      });
                    },
                    child: Center(
                      child: Container(
                        padding: const EdgeInsets.symmetric(
                            horizontal: 4, vertical: 2),
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(4),
                            border: Border.all(
                                color: isGif
                                    ? hexColor('#2F3240')
                                    : Colors.transparent)),
                        child: Icon(
                          Icons.gif,
                          size: 32,
                          color: hexColor('#2F3240'),
                        ),
                      ),
                    ),
                  ),
                ),
                Expanded(
                  child: InkWell(
                    onTap: () {
                      arckipelHaptic();
                      setState(() {
                        isGif = false;
                      });
                    },
                    child: Center(
                      child: Container(
                        padding: const EdgeInsets.symmetric(
                            horizontal: 8, vertical: 6),
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(4),
                            border: Border.all(
                                color: isGif
                                    ? Colors.transparent
                                    : hexColor('#2F3240'))),
                        child: Icon(
                          Icons.emoji_emotions,
                          size: 22,
                          color: hexColor('#2F3240'),
                        ),
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ),
          Divider(
            height: 30,
            thickness: 2.5,
            color: _backgroundColor,
          ),
          Expanded(
            child: AnimatedSwitcher(
              duration: const Duration(milliseconds: 200),
              child: isGif ? const _WidgetGifs() : const _WidgetEmojis(),
            ),
          )
        ],
      ),
    );
  }
}

class _WidgetGifs extends StatefulWidget {
  const _WidgetGifs({Key? key}) : super(key: key);

  @override
  __WidgetGifsState createState() => __WidgetGifsState();
}

class __WidgetGifsState extends State<_WidgetGifs> {
  late final ScrollController _scrollController = ScrollController();

  final GiphyClient client = GiphyClient(apiKey: GIPHY_API_KEY, randomId: '');
  final List<dynamic> results = [];

  final Axis _scrollDirection = Axis.vertical;
  late final int _crossAxisCount =
      (MediaQuery.of(context).size.width / _gifWidth).round();
  final double _spacing = 8.0;
  late final double _gifWidth = 200;
  late final int _limit = _crossAxisCount *
          ((MediaQuery.of(context).size.height - 30) / _gifWidth).round() +
      10;
  bool _isFetching = false;
  GiphyCollection? _collectionGiphy;

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      _scrollController.addListener(() {
        if ((_scrollController.position.extentAfter < 200) && !_isFetching) {
          _fetch();
        }
      });
      _fetch();
    });
  }

  _fetch([String? keyword]) async {
    if (_isFetching) return;
    _isFetching = true;
    if (keyword != null && keyword.trim().isNotEmpty) {
      results.clear();
      _collectionGiphy = await client.search(keyword, offset: 0, limit: _limit);
    } else {
      _collectionGiphy =
          await client.trending(offset: results.length, limit: _limit);
    }
    if (mounted) {
      setState(() {
        _isFetching = false;
        List data = [];
        data.addAll(_collectionGiphy?.data ?? []);
        data.shuffle();
        results.addAll(data);
      });
    }
    // results.forEach((element) {
    //   log('Tener result: ${element.toJson()}');
    // });
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.fromLTRB(16, 12, 16, 12),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Container(
            height: 38,
            decoration: BoxDecoration(
              color: _element,
              borderRadius: BorderRadius.circular(99),
            ),
            child: Center(
              child: Row(
                children: [
                  const SizedBox(
                    width: 16,
                  ),
                  Icon(
                    Icons.search,
                    color: hexColor('#2F3240'),
                  ),
                  Expanded(
                    child: TextFormField(
                        maxLines: 1,
                        onChanged: _fetch,
                        style: w300TextStyle().copyWith(fontSize: 12),
                        decoration: InputDecoration(
                          isCollapsed: true,
                          hintText: 'searchgif'.tr,
                          contentPadding:
                              const EdgeInsets.symmetric(horizontal: 12),
                          hintStyle: w300TextStyle().copyWith(
                              color: hexColor('#2F3240'),
                              fontSize: 12,
                              fontStyle: FontStyle.italic),
                          border: InputBorder.none,
                        )),
                  ),
                ],
              ),
            ),
          ),
          const SizedBox(
            height: 24,
          ),
          Expanded(
            child: _buildGifs(),
          ),
        ],
      ),
    );
  }

  Widget _buildGifs() {
    return results.isEmpty
        ? Center(child: Lottie.asset('assets/lotties/request.json', width: 120))
        : MasonryGridView.count(
            scrollDirection: _scrollDirection,
            controller: _scrollController,
            itemCount: results.length,
            crossAxisCount: _crossAxisCount,
            mainAxisSpacing: _spacing,
            crossAxisSpacing: _spacing,
            itemBuilder: (ctx, index) {
              return _item(index);
            },
            // staggeredTileBuilder: (idx) => StaggeredTile.fit(1),
          );
  }

  Widget _item(int index) {
    GiphyGif gif = results[index];
    double height = double.parse(gif.images!.fixedWidth.height) *
        (_gifWidth / double.parse(gif.images!.fixedWidth.width));
    return ClipRRect(
      borderRadius: BorderRadius.circular(10.0),
      child: InkWell(
        onTap: () {
          arckipelHaptic();
          Navigator.pop(
              context, WidgetGifsResult(true, gif.images!.fixedWidth.webp!));
        },
        child: ExtendedImage.network(gif.images!.fixedWidth.webp!,
            cache: true,
            gaplessPlayback: true,
            fit: BoxFit.fill,
            headers: const {'accept': 'image/*'}, loadStateChanged: (state) {
          return AnimatedSwitcher(
            duration: const Duration(milliseconds: 350),
            child: case2(
                state.extendedImageLoadState,
                {
                  LoadState.loading: Container(
                    color: _backgroundColor,
                    width: _gifWidth,
                    height: height,
                    child: Center(
                      child: Lottie.asset('assets/lotties/gifLoader.json',
                          package: '_private_chat', width: _gifWidth / 2),
                    ),
                  ),
                  LoadState.completed: ExtendedRawImage(
                    image: state.extendedImageInfo?.image,
                    width: _gifWidth,
                    fit: BoxFit.fill,
                  ),
                  LoadState.failed: Container(
                    color: _backgroundColor,
                    width: _gifWidth,
                    height: height,
                    child: const Center(
                      child: Icon(Icons.info),
                    ),
                  ),
                },
                Container(
                  color: _backgroundColor,
                  width: _gifWidth,
                  height: height,
                  child: Center(
                    child: Lottie.asset('assets/lotties/gifLoader.json',
                        package: '_private_chat', width: _gifWidth / 2),
                  ),
                )),
          );
        }),
      ),
    );
  }

  TValue? case2<TOptionType, TValue>(
    TOptionType selectedOption,
    Map<TOptionType, TValue> branches, [
    TValue? defaultValue,
  ]) {
    if (!branches.containsKey(selectedOption)) {
      return defaultValue;
    }

    return branches[selectedOption];
  }
}

class _WidgetEmojis extends StatefulWidget {
  const _WidgetEmojis({Key? key}) : super(key: key);

  @override
  __WidgetEmojisState createState() => __WidgetEmojisState();
}

class __WidgetEmojisState extends State<_WidgetEmojis> {
  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.fromLTRB(16, 12, 16, 12),
      child: GridView.count(
        crossAxisCount: 5,
        mainAxisSpacing: 20,
        crossAxisSpacing: 20,
        children: chatEmojis2
            .map((e) => GestureDetector(
                onTap: () {
                  Navigator.pop(context, WidgetGifsResult(false, e));
                },
                child: Lottie.asset(assetlottie('emoji2/$e'),
                    package: '_private_chat')))
            .toList(),
      ),
    );
  }
}
